@@ -49,6 +49,7 @@ except ImportError:
 from absl import app
 from absl import flags
 from absl import logging
+from absl.testing import _pretty_print_reporter
 from absl.testing import xml_reporter
 from absl.third_party import unittest3_backport
 import six
@@ -1103,6 +1104,9 @@ class TestCase(unittest3_backport.TestCase):
     the location within the structures where the first difference is found.
     This may be helpful when comparing large structures.
 
+    Mixed Set types are supported. Mixed Mapping types are supported, but the
+    order of the keys will not be considered in the comparison.
+
     Args:
       a: The first structure to compare.
       b: The second structure to compare.
@@ -1221,10 +1225,24 @@ def _sorted_list_difference(expected, actual):
   return missing, unexpected
 
 
+def _are_both_of_integer_type(a, b):
+  return isinstance(a, six.integer_types) and isinstance(b, six.integer_types)
+
+
+def _are_both_of_set_type(a, b):
+  return isinstance(a, collections.Set) and isinstance(b, collections.Set)
+
+
+def _are_both_of_mapping_type(a, b):
+  return isinstance(a, collections.Mapping) and isinstance(
+      b, collections.Mapping)
+
+
 def _walk_structure_for_problems(a, b, aname, bname, problem_list):
   """The recursive comparison behind assertSameStructure."""
   if type(a) != type(b) and not (  # pylint: disable=unidiomatic-typecheck
-      isinstance(a, six.integer_types) and isinstance(b, six.integer_types)):
+      _are_both_of_integer_type(a, b) or _are_both_of_set_type(a, b) or
+      _are_both_of_mapping_type(a, b)):
     # We do not distinguish between int and long types as 99.99% of Python 2
     # code should never care.  They collapse into a single type in Python 3.
     problem_list.append('%s is a %r but %s is a %r' %
@@ -1232,7 +1250,18 @@ def _walk_structure_for_problems(a, b, aname, bname, problem_list):
     # If they have different types there's no point continuing
     return
 
-  if isinstance(a, collections.Mapping):
+  if isinstance(a, collections.Set):
+    for k in a:
+      if k not in b:
+        problem_list.append(
+            '%s has %r but %s does not' % (aname, k, bname))
+    for k in b:
+      if k not in a:
+        problem_list.append('%s lacks %r but %s has it' % (aname, k, bname))
+
+  # NOTE: a or b could be a defaultdict, so we must take care that the traversal
+  # doesn't modify the data.
+  elif isinstance(a, collections.Mapping):
     for k in a:
       if k in b:
         _walk_structure_for_problems(
@@ -1669,7 +1698,7 @@ def _run_and_get_tests_result(argv, args, kwargs, xml_test_runner_class):
         # File exists error can occur with concurrent tests
         if e.errno != errno.EEXIST:
           raise
-    if sys.version_info.major == 2:
+    if six.PY2:
       xml_output = open(xml_output_file, 'w')
     else:
       xml_output = open(xml_output_file, 'w', encoding='utf-8')
@@ -1687,6 +1716,8 @@ def _run_and_get_tests_result(argv, args, kwargs, xml_test_runner_class):
     if kwargs.get('testRunner') is None:
       kwargs['testRunner'] = xml_test_runner_class
     kwargs['testRunner'].set_default_xml_stream(xml_output)
+  elif kwargs.get('testRunner') is None:
+    kwargs['testRunner'] = _pretty_print_reporter.TextTestRunner
 
   # Make sure tmpdir exists.
   if not os.path.isdir(FLAGS.test_tmpdir):
